import {
  Loader,
  Metadata,
  Http,
  IEventParam,
  IDictionary,
} from '@zeedhi/core';
  
import {
  Grid, Tabs, Form, TreeGridEditable,
} from '@zeedhi/common';

export class Pla01305ConsultaNutrientesController {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public filterNutrientesGrid() {
    const filter: IDictionary = [];
    let gridConsultaNutrientes: Grid;
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRowTree = plateTree.datasource.currentRow;
    const formUnidade = Metadata.getInstance<Form>('formUnidade').value;
    const cdfilial = formUnidade.UNIDADEFORM.CDFILIAL;
    
    let cdFiliCoast: any;
    let cdServFili: any;
    let cdTipoCard: any;
    let cdPadrQual: any;
    let cdRegiao: any;
    let tipoRec = 'P';
    let central: any;

    const tab = Metadata.getInstance<Tabs>('tabPratos');
    if (tab.activeTab === 4) {
      gridConsultaNutrientes = Metadata.getInstance('CONSULTANUTRIENTES-Local');
      const formLocal = Metadata.getInstance<Form>('FORM_LOCAL').value;
      cdFiliCoast = formLocal.BRANCH_LOCAL.CDFILIAL;
      tipoRec = 'F';
    } else if (tab.activeTab === 5) {
      gridConsultaNutrientes = Metadata.getInstance('CONSULTANUTRIENTES-Servico');
      const formService = Metadata.getInstance<Form>('FORM_SERVICE').value;
      cdFiliCoast = formService.BRANCH_SERVICE.CDFILIAL;
      cdServFili = formService.SERVICE.CDSERVFILI;
      tipoRec = 'S';
    } else if (tab.activeTab === 3) {
      gridConsultaNutrientes = Metadata.getInstance('CONSULTANUTRIENTES-TipoCard');
      const formTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD').value;
      cdTipoCard = formTpCard.TIPOCARD.CDTIPOCARD;
      tipoRec = 'TC';
      central = this.appController.getCdCentral();
    } else if (tab.activeTab === 6) {
      gridConsultaNutrientes = Metadata.getInstance('CONSULTANUTRIENTES-PadrQual');
      const formPadrQual = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE').value;
      cdPadrQual = formPadrQual.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
      tipoRec = 'Q';
      central = this.appController.getCdCentral();
    } else if (tab.activeTab === 7) {
      gridConsultaNutrientes = Metadata.getInstance('CONSULTANUTRIENTES-Regiao');
      const formRegiao = Metadata.getInstance<Form>('FORM_REGIAO').value;
      cdPadrQual = formRegiao.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
      cdRegiao = formRegiao.NMREGIAO.CDREGIAO;
      tipoRec = 'R';
      central = this.appController.getCdCentral();
    } else {
      gridConsultaNutrientes = Metadata.getInstance('CONSULTANUTRIENTES-Padrao');
    }
    
    cdFiliCoast = cdFiliCoast === undefined ? null : cdFiliCoast;
    cdServFili = cdServFili === undefined ? null : cdServFili;
    cdTipoCard = cdTipoCard === undefined ? null : cdTipoCard;
    cdPadrQual = cdPadrQual === undefined ? null : cdPadrQual;
    cdRegiao = cdRegiao === undefined ? null : cdRegiao;
    central = central === undefined ? null : central;

    this.utilities.onMountedFilterFind(filter, 'CDFILAUXPRAT', '=', currentRowTree.CDFILAUXPRAT);
    this.utilities.onMountedFilterFind(filter, 'CDPRATO', '=', currentRowTree.CDPRATO);
    this.utilities.onMountedFilterFind(filter, 'CDSERVFILI', '=', cdServFili);
    this.utilities.onMountedFilterFind(filter, 'CDFILIAL', '=', cdFiliCoast);
    this.utilities.onMountedFilterFind(filter, 'CDFILIPROD', '=', cdfilial);
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', central);
    this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', cdTipoCard);
    this.utilities.onMountedFilterFind(filter, 'NRPADRAOQUALIDADE', '=', cdPadrQual);
    this.utilities.onMountedFilterFind(filter, 'CDREGIAO', '=', cdRegiao);
    this.utilities.onMountedFilterFind(filter, 'TIPOREC', '=', tipoRec);
    gridConsultaNutrientes.datasource.filter = filter;
    gridConsultaNutrientes.datasource.get();
  }
  
  public calcularNDPCAL() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRowTree = plateTree.datasource.currentRow;
    const formUnidade = Metadata.getInstance<Form>('formUnidade').value;
    const cdfilial = formUnidade.UNIDADEFORM.CDFILIAL;
    
    let cdFiliCoast: any;
    let cdServFili: any;
    let cdTipoCard: any;
    let cdPadrQual: any;
    let cdRegiao: any;
    let merenda: any;
    let central: any;
   
    let rowForm: any = '';
    let colForm: any = '';
    let formNutrientes: any = '';
    let ndpcal: any = '';
    
    let tabName = '';
    
    const tab = Metadata.getInstance<Tabs>('tabPratos');
    if (tab.activeTab === 3) {
      if (document.getElementsByName('CONSULTANUTRIENTES-TipoCard').length > 0) {
        tabName = 'TipoCard';
        const formTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD').value;
        cdTipoCard = formTpCard.TIPOCARD?.CDTIPOCARD;
        if (cdTipoCard === undefined) {
          this.utilities.showAlert('alert', 'pme::tipoCardapioDeveSerInformado', 'error');
          const gridTpCard = Metadata.getInstance<Grid>('CONSULTANUTRIENTES-TipoCard');
          gridTpCard.datasource.data = [];
          return;
        }
        merenda = 'S';
        central = this.appController.getCdCentral();
        
        rowForm = Metadata.getInstance('rowFormNutrientes-TipoCard');
        colForm = rowForm.getChildInstance('colNutrientes-TipoCard');
        formNutrientes = colForm.getChildInstance('formNutrientes-TipoCard');
        ndpcal = formNutrientes.getChildInstance('NDPCALResult');
      } else return;
    } else if (tab.activeTab === 4) {
      if (document.getElementsByName('CONSULTANUTRIENTES-Local').length > 0) {
        tabName = 'Local';
        const formLocal = Metadata.getInstance<Form>('FORM_LOCAL').value;
        cdFiliCoast = formLocal.BRANCH_LOCAL?.CDFILIAL;
        if (cdFiliCoast === undefined) {
          this.utilities.showAlert('alert', 'pme::unidDeveSerInformado', 'error');
          const gridLocal = Metadata.getInstance<Grid>('CONSULTANUTRIENTES-Local');
          gridLocal.datasource.data = [];
          return;
        }
        
        rowForm = Metadata.getInstance('rowFormNutrientes-Local');
        colForm = rowForm.getChildInstance('colNutrientes-Local');
        formNutrientes = colForm.getChildInstance('formNutrientes-Local');
        ndpcal = formNutrientes.getChildInstance('NDPCALResult');
      } else return;
    } else if (tab.activeTab === 5) {
      if (document.getElementsByName('CONSULTANUTRIENTES-Servico').length > 0) {
        tabName = 'Servico';
        const formService = Metadata.getInstance<Form>('FORM_SERVICE').value;
        cdFiliCoast = formService.BRANCH_SERVICE?.CDFILIAL;
        cdServFili = formService.SERVICE?.CDSERVFILI;
        
        rowForm = Metadata.getInstance('rowFormNutrientes-Servico');
        colForm = rowForm.getChildInstance('colNutrientes-Servico');
        formNutrientes = colForm.getChildInstance('formNutrientes-Servico');
        ndpcal = formNutrientes.getChildInstance('NDPCALResult');
        
        if (cdServFili === undefined) {
          this.utilities.showAlert('alert', 'pme::unidEservDevemSerInformados', 'error');
          const gridServico = Metadata.getInstance<Grid>('CONSULTANUTRIENTES-Servico');
          gridServico.datasource.data = [];
          return;
        }
      } else return;
    } else if (tab.activeTab === 2) {
      if (document.getElementsByName('CONSULTANUTRIENTES-Padrao').length > 0) {
        tabName = 'Padrao';
        rowForm = Metadata.getInstance('rowFormNutrientes-Padrao');
        colForm = rowForm.getChildInstance('colNutrientes-Padrao');
        formNutrientes = colForm.getChildInstance('formNutrientes-Padrao');
        ndpcal = formNutrientes.getChildInstance('NDPCALResult');
      } else return;
    } else if (tab.activeTab === 6) {
      if (document.getElementsByName('CONSULTANUTRIENTES-PadrQual').length > 0) {
        tabName = 'PadrQual';
        const formFiltroPadrQual = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE');
        cdPadrQual = formFiltroPadrQual.value.NMPADRAOQUALIDADE?.NRPADRAOQUALIDADE;
        
        rowForm = Metadata.getInstance('rowFormNutrientes-PadrQual');
        colForm = rowForm.getChildInstance('colNutrientes-PadrQual');
        formNutrientes = colForm.getChildInstance('formNutrientes-PadrQual');
        ndpcal = formNutrientes.getChildInstance('NDPCALResult');
        
        if (cdPadrQual === undefined) {
          this.utilities.showAlert('alert', 'pme::padraoQualidadeDeveSerInformado', 'error');
          const gridqual = Metadata.getInstance<Grid>('CONSULTANUTRIENTES-PadrQual');
          gridqual.datasource.data = [];
          return;
        }
      } else return;
    } else if (tab.activeTab === 7) {
      if (document.getElementsByName('CONSULTANUTRIENTES-Regiao').length > 0) {
        tabName = 'Regiao';
        const formFiltroRegiao = Metadata.getInstance<Form>('FORM_REGIAO');
        cdPadrQual = formFiltroRegiao.value.NMPADRAOQUALIDADE?.NRPADRAOQUALIDADE;
        cdRegiao = formFiltroRegiao.value.NMREGIAO?.CDREGIAO;
        
        rowForm = Metadata.getInstance('rowFormNutrientes-Regiao');
        colForm = rowForm.getChildInstance('colNutrientes-Regiao');
        formNutrientes = colForm.getChildInstance('formNutrientes-Regiao');
        ndpcal = formNutrientes.getChildInstance('NDPCALResult');
        
        if (cdPadrQual === undefined || cdRegiao === undefined) {
          this.utilities.showAlert('alert', 'pme::regiaoDeveSerInformado', 'error');
          const gridRegiao = Metadata.getInstance<Grid>('CONSULTANUTRIENTES-Regiao');
          gridRegiao.datasource.data = [];
          return;
        }
      } else return;
    }
    
    cdFiliCoast = cdFiliCoast === undefined ? null : cdFiliCoast;
    cdServFili = cdServFili === undefined ? null : cdServFili;
    cdTipoCard = cdTipoCard === undefined ? null : cdTipoCard;
    cdPadrQual = cdPadrQual === undefined ? null : cdPadrQual;
    cdRegiao = cdRegiao === undefined ? null : cdRegiao;
    central = central === undefined ? null : central;
    merenda = merenda === undefined ? 'N' : merenda;
    
    const row = {
      CDFILIAL: cdfilial,
      CDFILIPROC: cdFiliCoast,
      CDSERVFILI: cdServFili,
      CDFILAUXPRAT: currentRowTree.CDFILAUXPRAT,
      CDPRATO: currentRowTree.CDPRATO,
      CDFILCENTPME: central,
      CDTIPOCARD: cdTipoCard,
      NRPADRAOQUALIDADE: cdPadrQual,
      CDREGIAO: cdRegiao,
      MERENDA: merenda,
    };
    const grid = Metadata.getInstance<Grid>(`CONSULTANUTRIENTES-${tabName}`);
    if (grid) grid.datasource.loading = true;
    Http.post('/calcularNDPCAL', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset.result.ndpcal[0];
      
      if (result) {
        const ndpcalNumber = parseInt(result, 10);
        // const ndpcal: Text = Metadata.getInstance('NDPCALResult');
        ndpcal.text = `${this.utilities.formatValuesToFixed(ndpcalNumber, 2)}${'%'}`;
        this.filterNutrientesGrid();
      }
    }).finally(() => {
      if (grid) grid.datasource.loading = false;
    });
  }
  
  public activateTab({ component }: IEventParam<Tabs>) {
    if (component.activeTab === 2) this.calcularNDPCAL();
  }
  
  public reload() {
    let grid: any;
    let form: any;
    const tabPratos: Tabs = Metadata.getInstance('tabPratos');
    
    if (tabPratos.activeTab === 2) grid = Metadata.getInstance('RECIPE_STANDARD');
    else if (tabPratos.activeTab === 3) {
      grid = Metadata.getInstance('RECIPE_TIPOCARD');
      form = Metadata.getInstance<Form>('FORM_TIPOCARD');
    } else if (tabPratos.activeTab === 4) {
      grid = Metadata.getInstance('RECIPE_LOCAL');
      form = Metadata.getInstance<Form>('FORM_LOCAL');
    } else if (tabPratos.activeTab === 5) {
      grid = Metadata.getInstance('RECIPE_SERVICE');
      form = Metadata.getInstance<Form>('FORM_SERVICE');
    } else if (tabPratos.activeTab === 6) {
      grid = Metadata.getInstance('RECIPE_PADRAOQUALIDADE');
      form = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE');
    } else if (tabPratos.activeTab === 7) {
      grid = Metadata.getInstance('RECIPE_REGION');
      form = Metadata.getInstance<Form>('FORM_REGIAO');
    }
    
    if (tabPratos.activeTab !== 2) {
      if (form.validate()) this.gridEditController.reload({ component: grid });
      else grid.datasource.data = [];
    } else this.gridEditController.reload({ component: grid });
  }
}
