import {
  Loader, Metadata, IDictionary, IEventParam, Http, I18n, Config,
} from '@zeedhi/core';

import {
  Form, Select, Grid, IDate, IGridEvent, GridEditable, TextInput, ModalService, Modal,
} from '@zeedhi/common';

import { PmeMask } from '@/config';

export class Pme12100FreqPratosPeriodo {
  public appController = Loader.getInstance('AppController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
    
  public crudController = Loader.getInstance('CrudController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public modalVigencia!: Modal;
  
  private selectedFreqPr = '';
  
  private descricao = '';
  
  private pratoIsVisible = false;
  
  private maxFreqGrPr = '';
  
  private selectedAgrup = '';
  
  private newValidVigencia = true;
  
  public setFilterTpCardapio({ component }: IDictionary) {
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
    this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const servico = Metadata.getInstance<Select>('SERVICO');
    const formGrupoPrato = Metadata.getInstance<Form>('formGrupoPrato');
    
    if (component.name === 'TIPOCARD') {
      servico.setValue('');
      this.selectedFreqPr = '';
      formGrupoPrato.value.describeCard = null;
      formGrupoPrato.value.finalDate = null;
      formGrupoPrato.value.initialDate = null;
      this.setFilterGrids();
    }
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const cdtipoCard: Select = Metadata.getInstance('TIPOCARD');
    let valueCard;
    
    if (cdtipoCard.returnObject === true && cdtipoCard.value !== null) valueCard = cdtipoCard.value.CDTIPOCARD;
    else valueCard = cdtipoCard.value;
    
    const conditions = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? valueCard : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterVigencia() {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const cdservfili = Metadata.getInstance<Select>('SERVICO');
    
    const filter = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? cdtipoCard.value : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cdservfili.value ? cdservfili.value : '',
      },
    ];
      
    if (document.getElementsByName('gridVigenciaFreqPrat').length > 0) {
      const gridVigenciaPrat = Metadata.getInstance<Grid>('gridVigenciaFreqPrat');
      if (gridVigenciaPrat) {
        gridVigenciaPrat.datasource.filter = filter;
        gridVigenciaPrat.datasource.get();
      }
    }
    
    if (document.getElementsByName('gridVigenciaFreqGrupPrat').length > 0) {
      const gridVigenciaGrPrat = Metadata.getInstance<Grid>('gridVigenciaFreqGrupPrat');
      if (gridVigenciaGrPrat) {
        gridVigenciaGrPrat.datasource.filter = filter;
        gridVigenciaGrPrat.datasource.get();
      }
    }
  }
  
  public resetDate({ component }: IEventParam<IDictionary>) {
    component.value = null;
    const fim: IDate = Metadata.getInstance<IDate>('finalDate');
    fim.value = null;
    this.selectedFreqPr = '';
  }
  
  public changeVigencia() {
    const tipoCard: IDictionary = Metadata.getInstance('TIPOCARD');
    const servico: IDictionary = Metadata.getInstance('SERVICO');
    const ini: IDate = Metadata.getInstance<IDate>('initialDate');
    const fim: IDate = Metadata.getInstance<IDate>('finalDate');
    if (ini.value !== null || (ini.value !== null && fim.value !== null && this.utilities.dateIsValid(ini, fim))) {
      if (tipoCard.value && servico.value) {
        const filter: IDictionary = [];
        this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', tipoCard.value);
        this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
        this.utilities.onMountedFilterFind(filter, 'CDSERVFILI', '=', servico.value);
        
        Http.post('/find/vigenciaFrequenciaPratos', { requestType: 'FilterData', filter }).then((response) => {
          const result = response.data.data;
          let desc = '';
          let nrFreq = '';
          let fimValue = fim.value;
          result.forEach((vigencia: IDictionary) => {
            if (vigencia.DTVIGENCIA === ini.value && vigencia.DTVIGENCIAFIM === fim.value) {
              desc = vigencia.DESCRICAO;
              nrFreq = vigencia.NRFREQPRATMESPAE;
              fimValue = vigencia.DTVIGENCIAFIM;
            }
          });
          this.descricao = desc;
          Metadata.getInstance<IDictionary>('description').value = desc;
          this.selectedFreqPr = nrFreq;
          fim.value = fimValue;
          if (this.selectedFreqPr === '' && !this.utilities.validateVigenciaDates(result, ini, fim)) {
            this.utilities.showAlert('alertError', 'pme::dataVigenciaIntercalada', 'error');
            this.newValidVigencia = false;
          } else {
            this.newValidVigencia = true;
          }
        }).finally(() => {
          this.setFilterGrids();
        });
      } else {
        // component.value = null;
        this.utilities.showAlert('alertError', 'pme::tpCardServNaoSelecionado', 'warning');
        this.setFilterGrids();
      }
    } else this.setFilterGrids();
  }
  
  public setVigencia({ row }: IGridEvent) {
    if (row) {
      const ini: IDate = Metadata.getInstance<IDate>('initialDate');
      const fim: IDate = Metadata.getInstance<IDate>('finalDate');
      ini.value = row.DTVIGENCIA;
      fim.value = row.DTVIGENCIAFIM;
      if (row.NRFREQPRATMESPAE) {
        this.selectedFreqPr = row.NRFREQPRATMESPAE;
      }
      const descricao: IDictionary = Metadata.getInstance('description');
      descricao.value = row.DESCRICAO;
      this.descricao = row.DESCRICAO;
      // this.setFilterGrids();
      this.changeVigencia();
      this.modalVigencia.hide();
    }
  }
  
  public setFilterGrids() {
    if (document.getElementsByName('gridFreqPorPrato').length > 0) {
      const gridFreqPorPrato: IDictionary = Metadata.getInstance('gridFreqPorPrato');
      const filter = [
        {
          name: 'NRFREQPRATMESPAE',
          operator: '=',
          value: this.selectedFreqPr,
        },
      ];
      gridFreqPorPrato.datasource.filter = filter;
      gridFreqPorPrato.datasource.get();
    }
    if (document.getElementsByName('gridFreqPorGrupoPratos').length > 0) {
      const gridFreqPorGrupoPratos: IDictionary = Metadata.getInstance('gridFreqPorGrupoPratos');
      const filterGrupo = [
        {
          name: 'NRFREQPRATMESPAE',
          operator: '=',
          value: this.selectedFreqPr,
        },
      ];
      
      gridFreqPorGrupoPratos.datasource.filter = filterGrupo;
      gridFreqPorGrupoPratos.datasource.get();
    }
  }
  
  private async createModalVigencia() {
    const modalPath = 'pme/restricoesContratuais/cadastros/modals/gridVigencias';
    this.modalVigencia = await ModalService.createFromJson(modalPath, true);
  }
  
  public async openVigenciaModal() {
    const tipoCard: IDictionary = Metadata.getInstance('TIPOCARD');
    const servico: IDictionary = Metadata.getInstance('SERVICO');
    if (tipoCard.value && servico.value) {
      if (!this.modalVigencia) {
        this.createModalVigencia().then(() => {
          this.modalVigencia.show();
          this.setFilterVigencia();
        });
      } else {
        this.modalVigencia.show();
        this.setFilterVigencia();
      }
    } else {
      this.utilities.showAlert('alertError', 'pme::tpCardServNaoSelecionado', 'warning');
    }
  }
  
  public closeModalVigencia() {
    this.modalVigencia.hide();
  }
  
  public savePorPrato() {
    const gridFreqPorPrato: GridEditable = Metadata.getInstance('gridFreqPorPrato');
    const form = Metadata.getInstance<Form>('formFreqPorPrato');
    const editedRows: IDictionary = gridFreqPorPrato.getEditedRows();
    if (this.selectedFreqPr === '') {
      if (!this.newValidVigencia) this.utilities.showAlert('alertError', 'pme::dataVigenciaIntercalada', 'error');
      else if (form.validate() && editedRows.length > 0) {
        const ini: IDate = Metadata.getInstance<IDate>('initialDate');
        const fim: IDate = Metadata.getInstance<IDate>('finalDate');
        const descricao: IDictionary = Metadata.getInstance('description');
        
        const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
        const cdservfili = Metadata.getInstance<Select>('SERVICO');
        
        Http.post('/returnCodFreqPrat', {}).then((response: IDictionary) => {
          const codigo = response.data.dataset.codigo[0].CODIGO;
          
          const vigencia = {
            NRFREQPRATMESPAE: codigo + 1,
            DTVIGENCIA: ini.value,
            DTVIGENCIAFIM: fim.value,
            DESCRICAO: descricao.value,
            CDFILCENTPME: this.appController.getCdCentral(),
            CDTIPOCARD: cdtipoCard.value,
            CDSERVFILI: cdservfili.value,
            __is_new: true,
          };
          
          this.crudController.saveRows([vigencia], 'vigenciaFrequenciaPratos', gridFreqPorPrato).then(() => {
            this.selectedFreqPr = vigencia.NRFREQPRATMESPAE;
            
            this.saveItFreqPrat(gridFreqPorPrato);
            // this.gridEditController.cancelEditing({ component: gridFreqPorPrato });
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          });
        });
      }
    } else {
      this.saveItFreqPrat(gridFreqPorPrato);
    }
  }
  
  private saveItFreqPrat(grid: GridEditable) {
    const form = Metadata.getInstance<Form>('formFreqPorPrato');
    const editedRows: IDictionary = grid.getEditedRows();
    if (form.validate() && editedRows.length > 0) {
      let valid = true;
      editedRows.forEach((row: IDictionary) => {
        row.NRFREQPRATMESPAE = this.selectedFreqPr;
        if (row.NRMINFREQPRATMES > row.NRMAXFREQPRATMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::minMesMaiorMaxMes', 'error');
        }
        if (row.NRMAXFREQPRATSEM > row.NRMAXFREQPRATMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::maxMesMenorMaxSem', 'error');
        }
        if (row.NRMAXFREQPRATSEM <= 0 || row.NRMAXFREQPRATMES <= 0 || row.NRMINFREQPRATMES <= 0) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::qtdMaiorZero', 'error');
        }
        if (row.NRMAXFREQPRATSEM > row.NRMINFREQPRATMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::maxSemMaiorMinMes', 'error');
        }
      });
      
      if (valid) {
        Http.post('/saveItFreqPrat', { editedRows }).then((result) => {
          if (result.data.length === 0) {
            this.setFilterGrids();
            this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
            this.gridEditController.cancelEditing({ component: grid });
          } else {
            const mensagem = result.data.dataset.mensagem[0];
            if (mensagem.indexOf('ERROR_PRATO_CADASTRADO_VIG') === 0) {
              this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_PRATO_CADASTRADO_VIG'), 'error');
            } else {
              this.utilities.showAlert('alert', `pme::${result.data}`, 'error');
            }
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        });
      }
    }
  }
  
  public savePorGrupoPrato() {
    const gridFreqPorGrupoPratos: GridEditable = Metadata.getInstance('gridFreqPorGrupoPratos');
    const form = Metadata.getInstance<Form>('formPorGrupoPratos');
    const editedRows: IDictionary = gridFreqPorGrupoPratos.getEditedRows();
    if (this.selectedFreqPr === '') {
      if (!this.newValidVigencia) this.utilities.showAlert('alertError', 'pme::dataVigenciaIntercalada', 'error');
      else if (form.validate() && editedRows.length > 0) {
        const ini: IDate = Metadata.getInstance<IDate>('initialDate');
        const fim: IDate = Metadata.getInstance<IDate>('finalDate');
        const descricao: IDictionary = Metadata.getInstance('description');
        
        const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
        const cdservfili = Metadata.getInstance<Select>('SERVICO');
        
        Http.post('/returnCodFreqPrat', {}).then((response: IDictionary) => {
          const codigo = response.data.dataset.codigo[0].CODIGO;
          
          const vigencia = {
            NRFREQPRATMESPAE: codigo + 1,
            DTVIGENCIA: ini.value,
            DTVIGENCIAFIM: fim.value,
            DESCRICAO: descricao.value,
            CDFILCENTPME: this.appController.getCdCentral(),
            CDTIPOCARD: cdtipoCard.value,
            CDSERVFILI: cdservfili.value,
            __is_new: true,
          };
          
          this.crudController.saveRows([vigencia], 'vigenciaFrequenciaPratos', gridFreqPorGrupoPratos).then(() => {
            this.selectedFreqPr = vigencia.NRFREQPRATMESPAE;
            
            this.saveItFreqGrPrat(gridFreqPorGrupoPratos);
            // this.gridEditController.cancelEditing({ component: gridFreqPorGrupoPratos });
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          });
        });
      }
    } else {
      this.saveItFreqGrPrat(gridFreqPorGrupoPratos);
    }
  }
  
  private saveItFreqGrPrat(grid: GridEditable) {
    const form = Metadata.getInstance<Form>('formPorGrupoPratos');
    const editedRows: IDictionary = grid.getEditedRows();
    if (form.validate() && editedRows.length > 0) {
      let valid = true;
      editedRows.forEach((row: IDictionary) => {
        row.NRFREQPRATMESPAE = this.selectedFreqPr;
        if (row.NRMINFREQGRPRATMES > row.NRMAXFREQGRPRATMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::minMesMaiorMaxMes', 'error');
        }
        if (row.NRMAXFREQGRPRATSEM > row.NRMAXFREQGRPRATMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::maxMesMenorMaxSem', 'error');
        }
        if (row.NRMAXFREQGRPRATSEM <= 0 || row.NRMAXFREQGRPRATMES <= 0 || row.NRMINFREQGRPRATMES <= 0) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::qtdMaiorZero', 'error');
        }
        if (row.NRMAXFREQGRPRATSEM > row.NRMINFREQGRPRATMES) {
          valid = false;
          this.utilities.showAlert('alert', 'pme::maxSemMaiorMinMes', 'error');
        }
      });
      if (valid) {
        Http.post('/saveItFreqGrPrat', { editedRows }).then((result) => {
          if (result.data.length === 0) {
            this.setFilterGrids();
            this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
            this.gridEditController.cancelEditing({ component: grid });
          } else {
            const mensagem = result.data.dataset.mensagem[0];
            if (mensagem.indexOf('ERROR_GRUPO_CADASTRADO_VIG') === 0) {
              this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_GRUPO_CADASTRADO_VIG'), 'error');
            } else {
              this.utilities.showAlert('alert', `pme::${result.data}`, 'error');
            }
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        });
      }
    }
  }
  
  public setFilterAgrup({ component, event }: IEventParam<Select>) {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const cdservfili = Metadata.getInstance<Select>('SERVICO');
    const gridFreqPorGrupoPratos: GridEditable = Metadata.getInstance('gridFreqPorGrupoPratos');
    // const nmagrup = gridFreqPorGrupoPratos.getChildInstance<Select>('NMAGRUPGRPRAT_editable_');
    const nmgrupo = gridFreqPorGrupoPratos.getChildInstance<Select>('NMGRUPOPRATOPAE_editable_');
    // const nmgrupoUnd = gridFreqPorGrupoPratos.getChildInstance<Select>('NMGRUPOPRATOPAE_editable_undefined');
    if (event && event.type === 'blur') this.selectedAgrup = component.value;
    
    const filter = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? cdtipoCard.value : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cdservfili.value ? cdservfili.value : '',
      },
      {
        name: 'NRAGRUPGRPRAT',
        operator: '=',
        value: this.selectedAgrup ? this.selectedAgrup : this.selectedAgrup,
      },
    ];
    
    nmgrupo.datasource.filter = filter;
    nmgrupo.datasource.search = '';
    nmgrupo.datasource.searchIn = [];
    nmgrupo.datasource.get().then((response) => {
      if (event && event.type === 'focus') component.datasource.data = response;
    });
  }
  
  public changeDescription({ component }: IEventParam<TextInput>) {
    if (this.selectedFreqPr !== '' && component.value !== this.descricao) {
      const ini: IDate = Metadata.getInstance<IDate>('initialDate');
      const fim: IDate = Metadata.getInstance<IDate>('finalDate');
      const descricao: IDictionary = Metadata.getInstance('description');
      const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
      const cdservfili = Metadata.getInstance<Select>('SERVICO');
      
      const vigencia = {
        NRFREQPRATMESPAE: this.selectedFreqPr,
        DTVIGENCIA: ini.value,
        DTVIGENCIAFIM: fim.value,
        DESCRICAO: descricao.value,
        CDFILCENTPME: this.appController.getCdCentral(),
        CDTIPOCARD: cdtipoCard.value,
        CDSERVFILI: cdservfili.value,
        __is_new: false,
      };
      
      this.crudController.saveRows([vigencia], 'vigenciaFrequenciaPratos').then(() => {
        this.descricao = descricao.value;
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public setFilterPrato({ component }: IEventParam<Select>) {
    const nivel: any = (Config.masks.plateMask as PmeMask).nivelMax;

    component.datasource.filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: null,
      },
      {
        name: 'CDNVPRATO',
        operator: '=',
        value: nivel,
      },
    ];
    component.datasource.get();
  }
  
  public newRow({ component }: IEventParam<Grid>) {
    const form = Metadata.getInstance<Form>('formGrupoPrato');
    if (form.validate()) this.gridEditController.addRow({ component });
  }
}
